/*
 * radiotrim -
 * pieced together by Jostein Kjønigsen for his own purposes and those only.
 * this is meant to work, not to be pretty or idiomatic c.
 */

#include <stdlib.h>
#include <stdio.h>

char* buffer;
char* source_name;
char* output_name;

unsigned long fileLen;

FILE *source;
FILE *output;

int bytesWritten;

// fixed point where baseband name is located
// http://forum.xda-developers.com/showpost.php?p=22741212&postcount=42
const unsigned int radioNameOffset = 0x009fefc0;

// fixed point where radio-data ends according to forums
// http://forum.xda-developers.com/showpost.php?p=22741212&postcount=42
const unsigned int radioSize = 0x00C00100;

int main( int argc, char *argv[] )
{
	printf("\nSamsung Galaxy Nexus I9250 radio-image trimmer.\n");

	if (argc != 3)
	{
		printf("Written by Jostein Kjonigsen for his own use only.\n");
		printf("This tool comes with NO warranties provided.\n\n");
		printf("usage: %s input.img output.img\n", argv[0] );
		exit(1);
	}

	source_name = argv[1];

	//Open file
	source = fopen(source_name, "rb");
	if (!source)
	{
		fprintf(stderr, "Unable to open file %s\n", source_name);
		exit(1);
	}

	fprintf(stdout, "%s opened successfully.\n", source_name);

	//Get file length
	fseek(source, 0, SEEK_END);
	fileLen=ftell(source);
	fseek(source, 0, SEEK_SET);

	if (fileLen < radioSize)
	{
		fprintf(stdout, "Possibly overtrimmed source-image!\n");
		fclose(source);
		exit(1);
	}

	//Allocate memory
	buffer=(char*)malloc(radioSize);
	if (!buffer)
	{
		fprintf(stderr, "Memory error!");
		fclose(source);
		exit(1);
	}

	//Read file contents into buffer
	fread(buffer, radioSize, 1, source);
	fclose(source);

	fprintf(stdout, "Detected baseband name: %s\n", (buffer+radioNameOffset));

	// output trimmed copy.
	output_name = argv[2];
	fprintf(stdout, "Creating trimmed copy %s (%i bytes).\n", output_name, radioSize);

	output = fopen(output_name, "w+b");
	if (!output)
	{
		fprintf(stderr, "Unable to open file %s\n", output_name);
		free(buffer);
		exit(1);
	}

	bytesWritten = fwrite(buffer, radioSize, 1, output);
	if (bytesWritten == radioSize)
	{
		fprintf(stderr,"Error writing file!\n");
		fclose(output);
		free(buffer);	
		exit(1);
	}

	fclose(output);
	free(buffer);

	return 0;
}
